<?php

	// first ever Meteotemplate OOP script :)

	class calculateET{
		public $Gsc = 0.0820; // solar constant, MJ * m-2 * min-1
		public $a = 0.23; // albedo or canopy reflection coefficient
		public $G = 0; // soil heat flux
		
		function __construct($Tavg, $RHavg, $f, $P, $fHeight, $elevation, $date, $lat, $solar){
			$this->Tmean = $Tavg;
			$this->RHavg = $RHavg;
			$this->f = $f;
			$this->P = $P;
			$this->fHeight = $fHeight;
			$this->elevation = $elevation;
			$this->date = $date;
			$this->lat = $lat;
			$this->solar = $solar * 24 * 3600 / 1000000; // solar radiation (MJ m-2 day-1)
			$this->sigma = 4.903 * pow(10,-9);
		}
		
		// calculate Rs from Ra, Tmax-Tmin
		public function set_Rs($deltaT)
			{
			$this->solar = 0.19 * sqrt($deltaT) * $this->get_Ra();
			}

		// wind speed at 2m above ground surface (m/s)
		public function get_f2(){ 
			$f2 = $this->f * (4.87 / log(67.8*$this->fHeight - 5.42));
			return $f2;
		}

		public function get_P(){
			if($this->P==="")
			    {$P = 101.3 * pow((293-0.0065*$this->elevation)/293,5.26);}
			else
			    {$P = $this->P;}
			return $P;
		}
		
		// slope of saturation vapor pressure curve
		public function get_D(){
			$D = (4098 * (0.6108 * exp((17.27*$this->Tmean)/($this->Tmean+237.3))))/(($this->Tmean+237.3)*($this->Tmean+237.3));
			return $D;
		}
		
		// psychrometric constant
		public function get_g(){
			$P = $this->get_P();
			$g = 0.665 * 0.001*$P; 
			return $g;
		}
		
		// mean saturation vapor pressure
		public function get_es(){
			$es = 0.6108 * exp((17.27*$this->Tmean)/($this->Tmean+237.3));
			return $es;
		}
		
		// actual vapor pressure derived from RH data
		public function get_ea(){
			$es = $this->get_es();
			$ea = ($es * ($this->RHavg/100)); 
			return $ea;
		}
		
		// vapor pressure deficit
		public function get_vpd(){
			$es = $this->get_es();
			$ea = $this->get_ea();
			$vpd = $es - $ea;
			return $vpd;
		}
		
		// format date 
		public function get_date(){
			$dateFormatted = explode("-",$this->date);
			$date['d'] = $dateFormatted[2];
			$date['m'] = $dateFormatted[1];
			$date['y'] = $dateFormatted[0];
			return $date;
		}
		
		// day of the year
		public function get_dayOfYear(){
			$date = $this->get_date();
			$J = date("z",strtotime($date['y']."/".$date['m']."/".$date['d'])); 
			return $J;
		}
		
		// inverse relative distance Earth-Sun
		public function get_dr(){
			$J = $this->get_dayOfYear();
			$dr = 1 + 0.033*cos((2*pi()/365 * $J));
			return $dr;
		}
		
		// solar declination (rad)
		public function get_declination(){
			$J = $this->get_dayOfYear();
			$d = 0.409 * sin(((2*pi())/365)*$J - 1.39); 
			return $d;
		} 
		
		// latitude to radians 
		public function get_latRad(){
			$j = pi()/180 * $this->lat;
			return $j;
		}
		
		// sunset hour angle (rad)
		public function get_ws(){
			$j = $this->get_latRad();
			$d = $this->get_declination();
			$ws = acos(-tan($j) * tan($d)); 
			return $ws;
		}
		
		// extraterrestrial radiation (MJ * m-2 * day-1)
		public function get_Ra(){
			$dr = $this->get_dr();
			$ws = $this->get_ws();
			$j = $this->get_latRad();
			$d = $this->get_declination();
			$Ra = (24*60/pi()) * $this->Gsc * $dr * ($ws * sin($j)*sin($d) + cos($j)*cos($d)*sin($ws)); 
			return $Ra;
		}
		
		// daylight hours 
		public function get_N(){
			$ws = $this->get_ws();
			$N = 24 / pi() * $ws;
			return $N;
		}
		
		// solar radiation calculated with Angstrom formula
		// recommended values if not measured as=0.25, bs=0.50
		
		// clear-sky solar radiation
		public function get_Rso(){
			$as = 0.25; // regression constant, expressing the fraction of extraterrestrial radiation reaching the earth on overcast days (n=0)
			$bs = 0.50;
			$Ra = $this->get_Ra();
			$Rso = ($as + $bs + 2 * 0.00001*$this->elevation)*$Ra;
			return $Rso;
		}
		
		// net solar shortwave radiation
		public function get_Rns(){
			$Rs = $this->solar; // $Rs = ($as + $bs * $n/$N)*$Ra; // solar radiation - if from daylight
			$Rns = (1-$this->a)*$Rs; 
			return $Rns;
		}
		
		// temperature conversion
		public function get_TavgK(){
			$TavgK = $this->Tmean + 273.16;
			return $TavgK;
		}
		
		// net longwave radiation
		public function get_Rnl(){
			$TavgK = $this->get_TavgK();
			$ea = $this->get_ea();
			$Rs = $this->solar;
			$Rso = $this->get_Rso();
			$sigmaTK4 = pow($TavgK,4) * $this->sigma;
			$Rnl = $sigmaTK4*(0.34 - 0.14 * sqrt($ea))*(1.35 * ($Rs/$Rso)-0.35); 
			return $Rnl;
		}
		
		// net radiation
		public function get_Rn(){
			$Rns = $this->get_Rns();
			$Rnl = $this->get_Rnl();
			$Rn = $Rns - $Rnl;
			return $Rn;
		}
		
		// evapotranspiration
		public function get_ET(){
			$Rn = $this->get_Rn();
			$D = $this->get_D();
			$g = $this->get_g();
			$f2 = $this->get_f2();
			$es = $this->get_es();
			$ea = $this->get_ea();
			$ETpart1 = 0.408 * ($Rn-$G) * $D / ($D + $g * (1 + 0.34*$f2));
			$ETpart2 = 900 / ($this->Tmean+273) * $f2 * ($es-$ea) * $g / ($D + $g * (1+0.34*$f2));
			$ET = $ETpart1 + $ETpart2; 
			if($ET<0){
				$ET=0;
			}
			return $ET;
		}
		
		// evapotranspiration
		public function get_ET1(){
			$Rn = $this->get_Rn();
			$D = $this->get_D();
			$g = $this->get_g();
			$f2 = $this->get_f2();
			$es = $this->get_es();
			$ea = $this->get_ea();
			$ETpart1 = 0.408 * ($Rn-$G) * $D / ($D + $g * (1 + 0.34*$f2));
			$ETpart2 = 900 / ($this->Tmean+273) * $f2 * ($es-$ea) * $g / ($D + $g * (1+0.34*$f2));
			return $ETpart1;
		}
		
		// evapotranspiration
		public function get_ET2(){
			$Rn = $this->get_Rn();
			$D = $this->get_D();
			$g = $this->get_g();
			$f2 = $this->get_f2();
			$es = $this->get_es();
			$ea = $this->get_ea();
			$ETpart1 = 0.408 * ($Rn-$G) * $D / ($D + $g * (1 + 0.34*$f2));
			$ETpart2 = 900 / ($this->Tmean+273) * $f2 * ($es-$ea) * $g / ($D + $g * (1+0.34*$f2));
			return $ETpart2;
		}
	}