<?php

	############################################################################
	# 	
	#	Meteotemplate
	# 	http://www.meteotemplate.com
	# 	Free website template for weather enthusiasts
	# 	Author: Jachym
	#           Brno, Czech Republic
	# 	First release: 2015
	#
	############################################################################
	#
	#	Plugin for updates via Ecowitt HTTP protocol by Raffaello Di Martino 
	#
	############################################################################
	#	Version and change log
	#
	# 	v1.0 - Sep 03, 2019
	# 		- initial release	
	#	v1.1 - Sep 05, 2019
	#		- Added support to CSV files
	#   v1.2 - Sep 30, 2019
	#	    - Added support for soilmoisture and pm25 sensors
	#   v1.3 - Oct 20, 2019
	#	    - Solved error for second WH31
	#   v1.4 - Jan 06, 2020
	#	    - baromabsin is changed with baromrelin
	#   v1.5 - Jan 15, 2020
	#	    - first version of temperature correction method based on Energy balance 
	#       - http://www.kwos.it/joomla/weather-monitoring/articoli/139-ecowitt-ws80-correzione-della-temperatura-rilevata
	#   v1.6 - Jan 26, 2020
	#	    - modification to the formula of temperature correction method based on Energy balance 
	#	v2.0 - May 11, 2020
	#	    - Battery status for sensors
	#	v2.1 - May 11, 2020
	#	    - Added WH65 Battery 
	#	v2.2 - Feb 01, 2021 (davidefa) 
	#	    - isset test on variables/fields before conversion
	#       . added 2 fields ( WH65BAT and WH25BAT ) used by WH2650A station to report external 7in1 sensors and internal termo/igro/press sensor
	#         ( ecowitt block can check/report these modules )
	#	v2.3 - Feb 04, 2021 (davidefa) 
	#       . added 2 fields ( Station and FW ) used by WH2650A station ( to report name of station and firmware revision )
	#         ( ecowitt block can check/report these field )
	#	v2.4 - Mar 03, 2021 (davidefa) 
	#        . fields ( WH65BAT, WH25BAT, Station and FW  ) exported to api.php
	#	v2.5 - Mar 19, 2021 (davidefa) 
	#        . send only defined fields
	#	v2.6 - May 1, 2021 (davidefa) 
	#        . added forwarding (in ecowitt/wu format)
	#        . added sensor remapping (in ecowitt/wu format)
	#	v2.6a - May 21, 2021 (davidefa) 
	#        . replaced file_get_contents with curl ( for compatibility with altervista )
	#	v2.6b - May 26, 2021 (davidefa)
	#        . corrected json and csv log
	#	v2.6c - Jun 29, 2021 (davidefa)
	#        . corrected 'password leaking' in log file ( api and ecowitt forward )
	#	v2.6d - Jul 15, 2021 (davidefa)
	#        . corrected 'password leaking' in log file ( wu forward )
	#	v2.7 - Jul 20, 2021 (davidefa)
	#        . added awekas format forwarding
	#	v2.8 - Jul 31, 2021 (davidefa)
	#        - added ambient weather protocol
	#	v2.9 - Oct 12, 2021 (davidefa)
	#        - corrected wn34 battery, added ws1900 battery
	#	v2.9 - Nov 25, 2021 (davidefa)
	#        - 
	#	v2.10 - Dic 03, 2021 (davidefa)
	#         - added ET0 calculation
	#	v2.11 - Jan 02, 2022 (davidefa)
	#         - fixes for compatibility with php 8
	#	v2.12 - Feb 13, 2022 (davidefa)
	#         - remove banks from parameters
	#	v3.0  - Jun 10, 2022 (davidefa)
	#         -
	#	v3.1  - Jan 30, 2023 (davidefa)
	#         - added realtime.txt generation option
	#         - added curl or require api.php call selection
	#	v3.2  - Feb 03, 2023 (davidefa)
	#         - added utc local time selection
	#         - added metric imperial unit selection
	#	v3.3  - Mar 16, 2023 (davidefa)
	#         - added 'weatherwizz' format
	#         - added fourth forwarder
	#	v3.4  - Mar 17, 2023 (davidefa)
	#         - added daily min/max calculation option ( for realtime.txt )
	#
	############################################################################


	define('__ROOT__', dirname(dirname(__FILE__)));

    // load settings
	if(!file_exists("../settings.php"))
	    {die("The settings file does not exist! Please go to your control panel and set up the netAtmo plugin.");}
	include("../settings.php");
	

    $base = "../../../";

    // load main info
    require($base."config.php");

    // load dependencies
    require($base."scripts/functions.php");

    // check access authorization
	if (isset($checkPassword) && $checkPassword == "password")
	    {
	    $password = $_GET['PASS'];
        // check if password is correct
        if($password!=$updatePassword)
	   	    {
            // if admin password provided accept, but notify
            if($password==$adminPassword)
                {echo "Authorized via admin password";}
            else
		  	    {die("Unauthorized");}
            }
        }
	if (isset($checkPassword) && $checkPassword == "passkey")
	    {
	    $password = $_REQUEST['PASSKEY'];
        // check if passkey is correct
        if($password!=$authPasskey)
	  	    {die("Unauthorized");}
        }
	if(!isset($use_curl)) {$use_curl = true;}	
	if(!isset($realtime_utc_time)) {$realtime_utc_time = false;}
	if(!isset($realtime_metric_unit)) {$realtime_metric_unit = true;}


	$weatherNSensors = 8;
	$ambientNSensors = 10;

    $nForwarders = 4;
	$forwardServer[0] = trim($forward_server);
	$forwardServer[1] = trim($forward_server_1);
	$forwardServer[2] = trim($forward_server_2);
	$forwardServer[3] = trim($forward_server_3);
	$forwardServer[4] = trim($forward_server_4);
	$forwardData[1] = trim($forward_data_1);
	$forwardData[2] = trim($forward_data_2);
	$forwardData[3] = trim($forward_data_3);
	$forwardData[4] = trim($forward_data_4);

	$forwardUsername[1] = $forward_username_1;
	$forwardUsername[2] = $forward_username_2;
	$forwardUsername[3] = $forward_username_3;
	$forwardUsername[4] = $forward_username_4;
	$forwardPassword[1] = $forward_password_1;
	$forwardPassword[2] = $forward_password_2;
	$forwardPassword[3] = $forward_password_3;
	$forwardPassword[4] = $forward_password_4;
	$forwardPort[1] = $forward_port_1;
	$forwardPort[2] = $forward_port_2;
	$forwardPort[3] = $forward_port_3;
	$forwardPort[4] = $forward_port_4;

	########### GET DATA ############

	# Settings: General
    ini_set( 'serialize_precision', -1 );       // so json_encode does not 'change roundness' ( for PHP >= 7.1 )
	$device = "auto";  # Use 'auto' for automatic name from PASSKEY else uses the name
	$forward_data = 1;  // has to be 1 to send data to meteotemplate api ( ...and you want to send data to meteotemplate api )
    $LogEcowitt = array();

	$json_data_logdir = $baseURL . "/plugins/ecowitt/";
	$csv_data_logdir = $baseURL . "/plugins/ecowitt/";
	$realtime_data_logdir = $baseURL . "/plugins/ecowitt/";
	$date_txt = date('Y-m-d');

	$weather_data_forward = array();
	$weather_data_wu_forward = array();
	$weather_data_wuext_forward = array();
	$weather_data_awekas_forward = array();
	$weather_data_weatherwizz_forward = array();

	# Convert HTTP POST variables to json
	if ( !isset($dataFormat) || (isset($dataFormat) && $dataFormat == "ecowitt") )
	    {
	    echo "ecowitt format<br>";
		$LogEcowitt['info'][] = "ecowitt format";
		$weather_data = $_POST;
		$weather_data_orig = $_POST;
		}
	# Convert HTTP GET variables to json
	if ( isset($dataFormat) && $dataFormat == "ambient" )
	    {
	    echo "ambient format<br>";
		$LogEcowitt['info'][] = "ambient format";
		$weather_data = $_GET;
		$weather_data_orig = $_GET;
		// rename sensor from ambient to ecowitt naming
		if (isset($weather_data['hourlyrainin']))  {@$weather_data['rainratein'] = $weather_data['hourlyrainin'] ;}	// wh51
		for ($i=1; $i<=$ambientNSensors; $i++)
		    {
		    if (isset($weather_data['soiltemp'.$i])) {@$weather_data['tf_ch'.$i] = $weather_data['soiltemp'.$i] ;}       	// wn34
			if (isset($weather_data['soilhum'.$i]))  {@$weather_data['soilmoisture'.$i] = $weather_data['soilhum'.$i] ;}	// wh51
			}
		}

    // log post and get data
    if(isset($_POST))
        {
        foreach($_POST as $urlParameter=>$value)
            {
            if( $urlParameter!="PASSWORD" && $urlParameter!="PASS" && !(($checkPassword == "passkey") && ($urlParameter=="PASSKEY")) )
                {$LogEcowitt['info'][] = "post ".$urlParameter.": ".$value;}
            }
        }
    if(isset($_GET))
        {
        foreach($_GET as $urlParameter=>$value)
            {
            if( $urlParameter!="PASSWORD" && $urlParameter!="PASS" && !(($checkPassword == "passkey") && ($urlParameter=="PASSKEY")) )
                {$LogEcowitt['info'][] = "get ".$urlParameter.": ".$value;}
            }
        }

    // 'vectorize' rename sensor ( meteotemplate format )
	$rename_sensor_fields = explode(";",$rename_sensor);
	foreach($rename_sensor_fields as $rename_sensor_field)
	    {
	    $renameSensor[] = explode("=",$rename_sensor_field);
	    $LogEcowitt['info'][] = "rename sensor ".$rename_sensor_field;    
	    }

    // 'vectorize' math with meteotemplate fields
    $mathMeteotemplate = array();
    $math_meteotemplate = trim($math_meteotemplate);
    if ($math_meteotemplate !== "")
        {
	    $math_meteotemplate_fields = explode(";",$math_meteotemplate);
	    foreach($math_meteotemplate_fields as $field)
	        {
	        $mathMeteotemplate[] = explode("=",$field);
	        $LogEcowitt['info'][] = "math meteotemplate ".$field;    
	        }
	    }

    // 'vectorize' ecowitt sensors ( map/remap )
	$remap_ecowitt_fields = explode(";",$remap_ecowitt);
	foreach($remap_ecowitt_fields as $remap_ecowitt_field)
	    {
	    $remapEcowitt[] = explode("=",$remap_ecowitt_field);
	    $LogEcowitt['info'][] = "remap meteotemplate ".$remap_ecowitt_field;    
	    }

    // 'vectorize' wu sensors ( map/remap )
	$remap_wu_fields = explode(";",$remap_wu);
	foreach($remap_wu_fields as $remap_wu_field)
	    {
	    $remapWU[] = explode("=",$remap_wu_field);
	    $LogEcowitt['info'][] = "remap wu ".$remap_wu_field;    
	    }

    // 'vectorize' awekas sensors ( map/remap )
	$remap_awekas_fields = explode(";",$remap_awekas);
	foreach($remap_awekas_fields as $remap_awekas_field)
	    {
	    $remapAwekas[] = explode("=",$remap_awekas_field);
	    $LogEcowitt['info'][] = "remap awekas ".$remap_awekas_field;    
	    }

    // 'vectorize' weatherwizz sensors ( map/remap )
	$remap_weatherwizz_fields = explode(";",$remap_weatherwizz);
	foreach($remap_weatherwizz_fields as $remap_weatherwizz_field)
	    {
	    $remapWeatherwizz[] = explode("=",$remap_weatherwizz_field);
	    $LogEcowitt['info'][] = "remap weatherwizz ".$remap_weatherwizz_field;    
	    }

	# Conversion factors
	$f_mph_kmh = 1.609344;
	$f_mph_kts = 0.868976;
	$f_mph_ms = 0.44704;
	$f_in_hpa = 33.863881;
	$f_in_mm = 25.4;
	
	# Get weather station identifier if requested
	if ( $device == "auto" ) {
    		$device = "weather_" . $weather_data['PASSKEY'];
	} else {
			$device = "weather_" . $device ;
	} 
	
	# Convert data
    # Temps
    # 2021-02-01 (davidefa)
    if (isset($weather_data['windchillf'])) {@$weather_data['windchillc'] = round( ( $weather_data['windchillf'] - 32 ) * 5 / 9, 2 );}
    if (isset($weather_data['tempf']))      {@$weather_data['tempc'] = round( ( $weather_data['tempf'] - 32 ) * 5 / 9, 2 );}
    if (isset($weather_data['tempinf']))    {@$weather_data['tempinc'] = round( ( $weather_data['tempinf'] - 32 ) * 5 / 9, 2 );}
    if (isset($weather_data['dewptf']))     {@$weather_data['dewptc'] = round( ( $weather_data['dewptf'] - 32 ) * 5 / 9, 2 );}	// not used??
    if (!isset($weather_data['dewptc']) && isset($weather_data['tempc']) && isset($weather_data['humidity']))     {@$weather_data['dewptc'] = dewpoint1($weather_data['tempc'], $weather_data['humidity']); @$weather_data['dewptf'] = $weather_data['dewptc'] * 1.8 + 32;}
    $LogEcowitt['info'][] = "dewptc ".$weather_data['dewptc']; 
    $LogEcowitt['info'][] = "tempc ".$weather_data['tempc']; 
    $LogEcowitt['info'][] = "humidity ".$weather_data['humidity']; 

	for ($i=1; $i<=max($ambientNSensors,$ecowittNSensors); $i++)
	    {
    	if (isset($weather_data['temp'.$i.'f']))  {@$weather_data['temp'.$i.'c'] = round( ( $weather_data['temp'.$i.'f'] - 32 ) * 5 / 9, 2 );} 	// wh30/wh31
		if (isset($weather_data['tf_ch'.$i]))     {@$weather_data['tc_ch'.$i] = round( ( $weather_data['tf_ch'.$i] - 32 ) * 5 / 9, 2 );}  		// wn34
	 	}

    # Speeds
    if (isset($weather_data['windgustmph']))  {@$weather_data['windgustkmh'] = round( $weather_data['windgustmph'] * $f_mph_kmh, 2 );}
    if (isset($weather_data['windspeedmph'])) {@$weather_data['windspeedkmh'] = round( $weather_data['windspeedmph'] * $f_mph_kmh, 2 );}
	if (isset($weather_data['windspeedmph'])) {@$weather_data['windspeedms'] = round( $weather_data['windspeedmph'] * $f_mph_ms, 2 );}
	if (isset($weather_data['maxdailygust'])) {@$weather_data['maxdailygustkmh'] = round( $weather_data['maxdailygust'] * $f_mph_kmh, 2 );}

    # Distances
    if (isset($weather_data['rainin']))        {@$weather_data['rainmm'] = round( $weather_data['rainin'] * $f_in_mm, 2 );}
    if (isset($weather_data['dailyrainin']))   {@$weather_data['dailyrainmm'] = round( $weather_data['dailyrainin'] * $f_in_mm, 2 );}
    if (isset($weather_data['weeklyrainin']))  {@$weather_data['weeklyrainmm'] = round( $weather_data['weeklyrainin'] * $f_in_mm, 2 );}
    if (isset($weather_data['monthlyrainin'])) {@$weather_data['monthlyrainmm'] = round( $weather_data['monthlyrainin'] * $f_in_mm, 2 );}
    if (isset($weather_data['yearlyrainin']))  {@$weather_data['yearlyrainmm'] = round( $weather_data['yearlyrainin'] * $f_in_mm, 2 );}
    if (isset($weather_data['rainratein']))    {@$weather_data['rainratemm'] = round( $weather_data['rainratein'] * $f_in_mm, 2 );}

    # Baros
    if (isset($weather_data['baromabsin'])) {@$weather_data['baromabshpa'] = round( $weather_data['baromabsin'] * $f_in_hpa, 2 );}
    if (isset($weather_data['baromrelin'])) {@$weather_data['baromrelhpa'] = round( $weather_data['baromrelin'] * $f_in_hpa, 2 );}

    # Date and time
    $dateUtcOrig = $weather_data['dateutc'];
    $weather_data['dateutc'] = $weather_data['dateutc'] . "Z";		// zulu time ( utc )
    $utcTime = strtotime($dateUtcOrig);
	$dateUtcOrigEncoded = str_replace(":","%3A",$dateUtcOrig);
	$dateUtcOrigEncoded = str_replace(" ","+",$dateUtcOrigEncoded);
    //$weather_data['dateutc'] = gmdate("Y-m-d\TH:i:s\Z");
	$this_tz_str = $stationTZ;
	$this_tz = new DateTimeZone($this_tz_str);
	$now = new DateTime("now", $this_tz);
	$offset = $this_tz->getOffset($now);

	if ($ws80_temperature_correction)
    	{
		# reading old data from database
		#
		# one value ago solar radiation
		$query_S_1 = "SELECT S FROM alldata ORDER BY DateTime DESC LIMIT 1";
		$result_S_1 = mysqli_query($con, $query_S_1);
		# two values ago solar radiation
		$query_S_2 = "SELECT S FROM alldata ORDER BY dateTime DESC LIMIT 1 OFFSET 1";
		$result_S_2 = mysqli_query($con, $query_S_2);
		# one value ago temperature
		$query_T_1 = "SELECT T FROM alldata ORDER BY DateTime DESC LIMIT 1";
		$result_T_1 = mysqli_query($con, $query_T_1);
		#
		# Fixed constants
		#
		$dm = 0.0009;
		$K = 0.0984;
		$cp = 29.3;
		$aslow = 0.300;
		$asnormal = 0.325;
		$ashigh = 0.335;
		$asultrahigh = 0.345;
		# Wind cannot be 0
		if ( $weather_data['windspeedms'] < 0.2 )
		  {$u = 0.2 ;}
		else 
		  {$u = $weather_data['windspeedms'];}
		#
		# as parameter estimation
		#
		if ( $weather_data['solarradiation'] < 1 )
		  {$S = 0.1;}
		elseif ( $weather_data['solarradiation'] > 352.3 ) 
		  {$S = 352.3;}
		else 
		  {$S = $weather_data['solarradiation'];}	
		
		if ( $S < $result_S_1 ) 
		    {
			if ( $result_S_1 < $result_S_2 ) 
			    {$as = $ashigh;}
			else 
			    {$as = $aslow;}	
		    } 
		else 
		    {
			if ( $S < $result_S_2 ) 
			    {$as = $aslow;}
			else 
			    {$as = $asnormal;}
		    }
		
		if ( ( $as == $ashigh ) || ( $as == $low ) ) 
		    {
			$diff_temp = $weather_data['tempc'] - $result_T_1;
			if ( $diff_temp > 0.3 ) 
			    {$as = $asultrahigh;}
		    }

		if ( round($weather_data['solarradiation'], 2) < 145 ) 
		  {$as = $asultrahigh*2.5;}		
		
		# Formula sections
		$sez1 = $as * $S;
		$sez2 = $u / $dm;
		$sez3 = sqrt($sez2);
		$sez4 = $cp * $K * $sez3;
		$sez5 = $sez1 / $sez4;
		$temp_corr = $weather_data['tempc'] - $sez5;
		
		# Original value of temperature from WS80
		@$weather_data['tempc_orig'] = $weather_data['tempc'];
		
		# Corrected value of temperature from WS80
		$diff_corr_temp = round($result_T_1, 2) - round($temp_corr, 2);
		if ( $diff_corr_temp > 0.7 ) 
		  {$temp_corr = $temp_corr+0.5;}
		if ( $diff_corr_temp < -0.7 ) 
		  {$temp_corr = $temp_corr-0.5;}
	
		# Corrected calue of temperature from WS80
		@$weather_data['tempc'] = round( $temp_corr, 2 );
	    }   // if ($ws80_temperature_correction)	
	
	# Forward data to meteotemplate server
	if ( $forward_data == 1 ) 
        {
		//@$weather_data_forward['U'] = strtotime( $weather_data['dateutc'] );
		//@$weather_data_forward['PASS'] = $updatePassword ;
		if (isset($weather_data['tempc'])) {@$weather_data_forward['T'] = $weather_data['tempc'] ;}
		if (isset($weather_data['humidity'])) {@$weather_data_forward['H'] = $weather_data['humidity'] ;}
		if (isset($weather_data['baromrelhpa'])) {@$weather_data_forward['P'] = $weather_data['baromrelhpa'] ;}
		if (isset($weather_data['windspeedkmh'])) {@$weather_data_forward['W'] = $weather_data['windspeedkmh'] ;}
		if (isset($weather_data['windgustkmh'])) {@$weather_data_forward['G'] = $weather_data['windgustkmh'] ;}
		if (isset($weather_data['winddir'])) {@$weather_data_forward['B'] = $weather_data['winddir'] ;}
		if (isset($weather_data['dailyrainmm'])) {@$weather_data_forward['R'] = $weather_data['dailyrainmm'] ;}
		if (isset($weather_data['rainratemm'])) {@$weather_data_forward['RR'] = $weather_data['rainratemm'] ;}
		if (isset($weather_data['solarradiation'])) {@$weather_data_forward['S'] = $weather_data['solarradiation'] ;}
		if (isset($weather_data['uv'])) {@$weather_data_forward['UV'] = $weather_data['uv'] ;}
		if (isset($weather_data['tempinc'])) {@$weather_data_forward['TIN'] = $weather_data['tempinc'] ;}
		if (isset($weather_data['humidityin'])) {@$weather_data_forward['HIN'] = $weather_data['humidityin'] ;}
		for ($i=1; $i<=max($ambientNSensors,$ecowittNSensors); $i++)
	        {
		    if (isset($weather_data['temp'.$i.'c']))       {@$weather_data_forward['T'.$i] = $weather_data['temp'.$i.'c'] ;}    		// wn30/wh31
		    if (isset($weather_data['humidity'.$i]))       {@$weather_data_forward['H'.$i] = $weather_data['humidity'.$i] ;}    		// wh31
		    if (isset($weather_data['tc_ch'.$i]))          {@$weather_data_forward['TS'.$i] = $weather_data['tc_ch'.$i] ;}       		// wn34
		    if (isset($weather_data['leafwetness_ch'.$i])) {@$weather_data_forward['LW'.$i] = $weather_data['leafwetness_ch'.$i] ;}  	// wn35
		    if (isset($weather_data['pm25_ch'.$i]))        {@$weather_data_forward['PP'.$i] = $weather_data['pm25_ch'.$i] ;}	        // wh41/wh43
		    if (isset($weather_data['soilmoisture'.$i]))   {@$weather_data_forward['SM'.$i] = $weather_data['soilmoisture'.$i] ;}		// wh51
			}
		if (isset($weather_data['co2']))            {@$weather_data_forward['CO2_1'] = $weather_data['co2'] ;}						// wh45
		if (isset($weather_data['pm25']))           {@$weather_data_forward['PP1'] = $weather_data['pm25'] ;}						// wh41/wh43  pm2.5 ( ambient )
		if (isset($weather_data['pm25_in']))        {@$weather_data_forward['PP2'] = $weather_data['pm25_in'] ;}					// wh45  pm2.5 ( ambient )
		if (isset($weather_data['pm10_in']))        {@$weather_data_forward['PP3'] = $weather_data['pm10_in'] ;}					// wh45  pm1.0 ( ambient )
		if (isset($weather_data['co2_in']))         {@$weather_data_forward['CO2_1'] = $weather_data['co2_in'] ;}					// wh45  ( ambient )
		if (isset($weather_data['lightning_num']))  {@$weather_data_forward['L'] = $weather_data['lightning_num']; }				// wh57
		if (isset($weather_data['lightning']))      {@$weather_data_forward['LD'] = $weather_data['lightning'] ;}
		if (isset($weather_data['lightning_time'])) {@$weather_data_forward['LT'] = $weather_data['lightning_time'] ;}
		if (isset($weather_data['lightning_day']))  {@$weather_data_forward['L'] = $weather_data['lightning_day']; }				// wh57  ( ambient )
		if (isset($weather_data['lightning_distance'])){@$weather_data_forward['LD'] = $weather_data['lightning_distance'] ;}

        // batteries
		if (isset($weather_data['wh25batt']))  {@$weather_data_forward['WH25BAT'] = $weather_data['wh25batt'];}						// wh32b indoor T/H/P
		if (isset($weather_data['wh26batt']))  {@$weather_data_forward['WH26BAT'] = $weather_data['wh26batt'];}						// wh32 external T/H
		if (isset($weather_data['wh40batt']))  {@$weather_data_forward['RBAT'] =    $weather_data['wh40batt'];}						// wh40 
		if (isset($weather_data['wh40batt']))  {@$weather_data_forward['WH40BAT'] = $weather_data['wh40batt'];}						// wh40
		if (isset($weather_data['battrain']))  {@$weather_data_forward['RBAT'] =    $weather_data['battrain'];}						// wh40 ( ambient ) ??
		if (isset($weather_data['wh65batt']))  {@$weather_data_forward['WH65BAT'] = $weather_data['wh65batt'];}                     // ws65 osprey sensor
		if (isset($weather_data['wh68batt']))  {@$weather_data_forward['WH68BAT'] = $weather_data['wh68batt'];}						// ws68 wind cup sensor
		if (isset($weather_data['wh80batt']))  {@$weather_data_forward['WH80BAT'] = $weather_data['wh80batt'];}                     // ws80 wind ultrasonic sensor
		if (isset($weather_data['ws1900batt'])){@$weather_data_forward['WS1900BAT'] = $weather_data['ws1900batt'];}					// ws1900 console
		for ($i=1; $i<=max($ambientNSensors,$ecowittNSensors); $i++)
	        {
		    if (isset($weather_data['batt'.$i]) )      {@$weather_data_forward['T'.$i.'BAT']  = $weather_data['batt'.$i] ;}         // wn30/wh31
		    if (isset($weather_data['tf_batt'.$i]))    {@$weather_data_forward['TS'.$i.'BAT'] = $weather_data['tf_batt'.$i] ;}      // wn34
		    if (isset($weather_data['leaf_batt'.$i]))  {@$weather_data_forward['LW'.$i.'BAT'] = $weather_data['leaf_batt'.$i] ;}    // wn35
		    if (isset($weather_data['pm25batt'.$i]))   {@$weather_data_forward['PP'.$i.'BAT'] = $weather_data['pm25batt'.$i];}      // wh41
		    if (isset($weather_data['soilbatt'.$i]))   {@$weather_data_forward['SM'.$i.'BAT'] = $weather_data['soilbatt'.$i];}      // wh51
		    if (isset($weather_data['battsm'.$i]))     {@$weather_data_forward['SM'.$i.'BAT'] = $weather_data['battsm'.$i];}     	// wh51  ( ambient )
			}
		if (isset($weather_data['co2_batt']))          {@$weather_data_forward['CO2_1BAT'] = $weather_data['co2_batt'] ;}  	        // wh45
		if (isset($weather_data['wh57batt']))          {@$weather_data_forward['LBAT'] =     $weather_data['wh57batt'];}            // wh57
		if (isset($weather_data['batt_lightning']))    {@$weather_data_forward['LBAT'] =     $weather_data['batt_lightning'];}      // wh57  ( ambient )
		if (isset($weather_data['batt_25']))           {@$weather_data_forward['PP1'] =      $weather_data['batt_25'];}         	// wh45  ( ambient ) ??
		if (isset($weather_data['batt_25in']))         {@$weather_data_forward['CO2_1BAT'] = $weather_data['batt_25in'] ;}  	    // wh41  ( ambient ) ??
		// ecowitt
		if ( !isset($dataFormat) || (isset($dataFormat) && $dataFormat == "ecowitt") )
		    {
			if (isset($weather_data['stationtype']))                                             // 2021-02-04 (davidefa)
		        {
                $weather_data['stationtype'] = str_replace(" ","_",$weather_data['stationtype']);
		    	$stationtype=explode("_",$weather_data['stationtype']);
		    	@$weather_data_forward['Station'] = $stationtype[0];
		    	@$weather_data_forward['FW'] = $stationtype[1];
		    	}
		    if ( $et0_calculate && ((date("G",$utcTime+$offset) == 23) && (date("i",$utcTime+$offset) >= 40) && (date("i",$utcTime+$offset) < 45)) )
		        {
	            require_once("evapoTranspCalculate.php");
		        @$weather_data_forward['ET'] = calcET0($defaultWindHeight,$defaultElevation,true,true,0); 
		        //$LogEcowitt['info'][] = "locale: ".date("Y-m-d H:i:s",$utcTime+$offset);
		        //$LogEcowitt['info'][] = "ore: ".date("G",$utcTime+$offset);
		        //$LogEcowitt['info'][] = "min: ".date("i",$utcTime+$offset);
		        }
		    }
		// ambient
		if ( isset($dataFormat) && $dataFormat == "ambient" )
		    {
			if (isset($weather_data['stationtype']))                                             // 2021-08-02 (davidefa)
		       {
		       $stationtype=explode("V",$weather_data['stationtype']);
		       @$weather_data_forward['Station'] = $stationtype[0];
		       @$weather_data_forward['FW'] = $stationtype[1];
		       }
		    }
		// rename sensor ( meteotemplate format )
		foreach($renameSensor as $rs)
		    { 
		    if (isset($weather_data_forward[$rs[1]])) {@$weather_data_forward[$rs[0]] = @$weather_data_forward[$rs[1]]; unset($weather_data_forward[$rs[1]]);} 
		    }

		// math meteotemplate fields
		foreach($mathMeteotemplate as $mm)
		    {
		    @$weather_data_forward[$mm[0]] = subst_eval($mm[1]); 
		    $LogEcowitt['info'][] = "math meteotemplate ".$mm[0]."=".$mm[1]."=".$weather_data_forward[$mm[0]];    
		    }

		// remap ecowitt fields
		foreach($remapEcowitt as $re)
		    { 
		    if (isset($weather_data[$re[1]])) {@$weather_data_forward[$re[0]] = $weather_data[$re[1]] ;} 
		    $LogEcowitt['info'][] = "remapping ecowitt ".$re[0]."<-".$re[1]."=".$weather_data[$re[1]];    
		    }

        // wu format
		if (isset($weather_data['dateutc']))        {@$weather_data_wu_forward['dateutc'] = $dateUtcOrigEncoded;}
		//if (isset($weather_data['dateutc']))        {@$weather_data_wu_forward['dateutc'] = "now";}
		if (isset($weather_data['baromrelin']))     {@$weather_data_wu_forward['baromin'] = $weather_data['baromrelin'];}
		if (isset($weather_data['tempinf']))        {@$weather_data_wu_forward['indoortempf'] = $weather_data['tempinf'];}
		if (isset($weather_data['humidityin']))     {@$weather_data_wu_forward['indoorhumidity'] = $weather_data['humidityin'];}
		if (isset($weather_data['tempf']))          {@$weather_data_wu_forward['tempf'] = $weather_data['tempf'];}
		if (isset($weather_data['humidity']))       {@$weather_data_wu_forward['humidity'] = $weather_data['humidity'];}
		if (isset($weather_data['winddir']))        {@$weather_data_wu_forward['winddir'] = $weather_data['winddir'];}
		if (isset($weather_data['windspeedmph']))   {@$weather_data_wu_forward['windspeedmph'] = $weather_data['windspeedmph'];}
		if (isset($weather_data['windgustmph']))    {@$weather_data_wu_forward['windgustmph'] = $weather_data['windgustmph'];}
		if (isset($weather_data['solarradiation'])) {@$weather_data_wu_forward['solarradiation'] = $weather_data['solarradiation'];}
		if (isset($weather_data['uv']))             {@$weather_data_wu_forward['UV'] = $weather_data['uv'];}
		if (isset($weather_data['hourlyrainin']))   {@$weather_data_wu_forward['rainin'] = $weather_data['hourlyrainin'];}
		if (isset($weather_data['dailyrainin']))    {@$weather_data_wu_forward['dailyrainin'] = $weather_data['dailyrainin'];}
		if (isset($weather_data['temp1f']))         {@$weather_data_wuext_forward['temp2f'] = $weather_data['temp1f'];}
		if (isset($weather_data['temp2f']))         {@$weather_data_wuext_forward['temp3f'] = $weather_data['temp2f'];}
		if (isset($weather_data['temp3f']))         {@$weather_data_wuext_forward['temp4f'] = $weather_data['temp3f'];}
		if (isset($weather_data['temp4f']))         {@$weather_data_wuext_forward['temp5f'] = $weather_data['temp4f'];}
		if (isset($weather_data['temp5f']))         {@$weather_data_wuext_forward['temp6f'] = $weather_data['temp5f'];}
		if (isset($weather_data['temp6f']))         {@$weather_data_wuext_forward['temp7f'] = $weather_data['temp6f'];}
		if (isset($weather_data['temp7f']))         {@$weather_data_wuext_forward['temp8f'] = $weather_data['temp7f'];}
		if (isset($weather_data['temp8f']))         {@$weather_data_wuext_forward['temp9f'] = $weather_data['temp8f'];}
		if (isset($weather_data['soilmoisture1']))  {@$weather_data_wuext_forward['soilmoisture']  = $weather_data['soilmoisture1'];}
		if (isset($weather_data['soilmoisture2']))  {@$weather_data_wuext_forward['soilmoisture2'] = $weather_data['soilmoisture2'];}
		if (isset($weather_data['soilmoisture3']))  {@$weather_data_wuext_forward['soilmoisture3'] = $weather_data['soilmoisture3'];}
		if (isset($weather_data['soilmoisture4']))  {@$weather_data_wuext_forward['soilmoisture4'] = $weather_data['soilmoisture4'];}
		if (isset($weather_data['soilmoisture5']))  {@$weather_data_wuext_forward['soilmoisture5'] = $weather_data['soilmoisture5'];}
		if (isset($weather_data['soilmoisture6']))  {@$weather_data_wuext_forward['soilmoisture6'] = $weather_data['soilmoisture6'];}
		if (isset($weather_data['soilmoisture7']))  {@$weather_data_wuext_forward['soilmoisture7'] = $weather_data['soilmoisture7'];}
		if (isset($weather_data['soilmoisture8']))  {@$weather_data_wuext_forward['soilmoisture8'] = $weather_data['soilmoisture8'];}
		if (isset($weather_data['stationtype']))    {@$weather_data_wuext_forward['softwaretype']  = $weather_data['stationtype'];}
		// remap wu fields
		foreach($remapWU as $rw)
		    { if (isset($weather_data[$rw[1]])) {@$weather_data_wu_forward[$rw[0]] = $weather_data[$rw[1]];} }

        // awekas format
		if (isset($weather_data['tempinc']))        {@$weather_data_awekas_forward['indoortempf'] = $weather_data['tempinc'] ;}
		if (isset($weather_data['humidityin']))     {@$weather_data_awekas_forward['indoorhumidity'] = $weather_data['humidityin'] ;}
		if (isset($weather_data['tf_ch1']))         {@$weather_data_awekas_forward['soiltemp1'] = $weather_data['tf_ch1'] ;}       // wn34
		if (isset($weather_data['tf_ch2']))         {@$weather_data_awekas_forward['soiltemp2'] = $weather_data['tf_ch2'] ;}      
		if (isset($weather_data['tf_ch3']))         {@$weather_data_awekas_forward['soiltemp3'] = $weather_data['tf_ch3'] ;}      
		if (isset($weather_data['tf_ch4']))         {@$weather_data_awekas_forward['soiltemp4'] = $weather_data['tf_ch4'] ;}      
		if (isset($weather_data['soilmoisture1']))  {@$weather_data_awekas_forward['soilmoisture1'] = $weather_data['soilmoisture1'] ;}
		if (isset($weather_data['soilmoisture2']))  {@$weather_data_awekas_forward['soilmoisture2'] = $weather_data['soilmoisture2'] ;}
		if (isset($weather_data['soilmoisture3']))  {@$weather_data_awekas_forward['soilmoisture3'] = $weather_data['soilmoisture3'] ;}
		if (isset($weather_data['soilmoisture4']))  {@$weather_data_awekas_forward['soilmoisture4'] = $weather_data['soilmoisture4'] ;}
		if (isset($weather_data['leafwetness_ch1'])) {@$weather_data_awekas_forward['leafwetness1'] = $weather_data['leafwetness_ch1'] ;}
		if (isset($weather_data['leafwetness_ch2'])) {@$weather_data_awekas_forward['leafwetness2'] = $weather_data['leafwetness_ch2'] ;}
		if (isset($weather_data['humidity1']))      {@$weather_data_awekas_forward['hum1'] = $weather_data['humidity1'] ;}
		if (isset($weather_data['humidity2']))      {@$weather_data_awekas_forward['hum2'] = $weather_data['humidity2'] ;}
		if (isset($weather_data['humidity3']))      {@$weather_data_awekas_forward['hum3'] = $weather_data['humidity3'] ;}
		if (isset($weather_data['humidity4']))      {@$weather_data_awekas_forward['hum4'] = $weather_data['humidity4'] ;}
		//if (isset($weather_data['pm25_ch1']))     {@$weather_data_awekas_forward['AqPM1'] = $weather_data['pm25_ch1'] ;}
		if (isset($weather_data['pm25_ch1']))       {@$weather_data_awekas_forward['AqPM2.5'] = $weather_data['pm25_ch1'] ;}
		//if (isset($weather_data['pm25_ch1']))     {@$weather_data_awekas_forward['AqPM10'] = $weather_data['pm25_ch1'] ;}
		$weather_data_awekas_forward['output'] = 'json';
		// remap awekas fields
		foreach($remapAwekas as $ra)
		    { if (isset($weather_data[$ra[1]])) {@$weather_data_awekas_forward[$ra[0]] = $weather_data[$ra[1]] ;} }
		
        // weatherwizz format
		if (isset($weather_data['tempinc']))        {@$weather_data_weatherwizz_forward['it'] = $weather_data['tempinc'] ;}
		if (isset($weather_data['humidityin']))     {@$weather_data_weatherwizz_forward['ih'] = $weather_data['humidityin'] ;}
		if (isset($weather_data['tempc']))          {@$weather_data_weatherwizz_forward['oc'] = $weather_data['tempc'];}
		if (isset($weather_data['humidity']))       {@$weather_data_weatherwizz_forward['h'] = $weather_data['humidity'];}
		if (isset($weather_data['baromrelhpa']))    {@$weather_data_weatherwizz_forward['ap'] = $weather_data['baromrelhpa'];}
		if (isset($weather_data['winddir']))        {@$weather_data_weatherwizz_forward['wd'] = $weather_data['winddir'];}
		if (isset($weather_data['windspeedkmh']))   {@$weather_data_weatherwizz_forward['ws'] = $weather_data['windspeedkmh'];}
		if (isset($weather_data['windgustkmh']))    {@$weather_data_weatherwizz_forward['g'] = $weather_data['windgustkmh'];}
		if (isset($weather_data['solarradiation'])) {@$weather_data_weatherwizz_forward['sr'] = $weather_data['solarradiation'];}
		if (isset($weather_data['uv']))             {@$weather_data_weatherwizz_forward['uv'] = $weather_data['uv'];}
		if (isset($weather_data['rainratemm']))     {@$weather_data_weatherwizz_forward['rr'] = $weather_data['rainratemm'];}
		if (isset($weather_data['dailyrainmm']))    {@$weather_data_weatherwizz_forward['r'] = $weather_data['dailyrainmm'];}
		if (isset($weather_data['dewptc']))         {@$weather_data_weatherwizz_forward['od'] = $weather_data['dewptc'];}
		if (isset($weather_data['windchillc']))     {@$weather_data_weatherwizz_forward['wc'] = $weather_data['windchillc'];}
		// remap weatherwizz fields
		foreach($remapWeatherwizz as $ra)
		    { if (isset($weather_data[$ra[1]])) {@$weather_data_weatherwizz_forward[$ra[0]] = $weather_data[$ra[1]] ;} }
		
		// forward to meteotemplate api.php...
		// ...using curl
		$wdf = "";
		foreach($weather_data_forward as $key => $value)
			{$wdf .= '&' . $key . '=' . $value;}
		if ($use_curl)
		    {
	    	$weather_data_url = $forward_server  . "?U=" . strtotime($weather_data['dateutc']) . "&PASS=" . $updatePassword . $wdf;
    		$weather_data_urlnp = $forward_server  . "?U=" . strtotime($weather_data['dateutc']) . $wdf;
	        $ch = curl_init();
	        curl_setopt($ch, CURLOPT_URL, $weather_data_url );
    	    curl_setopt($ch, CURLOPT_HEADER, false);
	        $out = curl_exec($ch);
	        $LogEcowitt['info'][] = "Api url -> ".$weather_data_urlnp;
    	    $LogEcowitt['info'][] = "Api result -> ".$out;
	        //echo "Api result -> ".$out;
            if (curl_error($ch)) {$LogEcowitt['info'][] = "Api error -> ".curl_error($ch);}
    	    curl_close($ch);
		    }
		else
		    {
		    // ...using require_once
            echo "Calling api.php date/time: ".date("Y-m-d H:i:s")."<br>";
            $LogEcowitt['info'][] = "Calling api.php date/time: ".date("Y-m-d H:i:s");
    	    //generateLogEcowitt();
            // call api.php
    	    $apiUpdate = 'ecowitt plugin';
		    $rawUpdate = $weather_data_forward;
		    $rawUpdate['U'] = strtotime($weather_data['dateutc']);
	        $LogEcowitt['info'][] = "Api call -> ".json_encode($rawUpdate);
		    $rawUpdate['PASS'] = $updatePassword;
            require_once($base."api.php");
            echo "<br>Exiting api.php date/time: ".date("Y-m-d H:i:s")."<br>";
            $LogEcowitt['info'][] = "Exiting api.php date/time: ".date("Y-m-d H:i:s");
    	    //echo "rawUpdate: "; print_r($rawUpdate); echo "<br>";
		    }
        }   // if ( $forward_data == 1 )  to meteotemplate server

    // generate realtime.txt ( CumulusMX format )
    if ($realtime_data_log)
        {
        // Date and time
        $dt = explode(" ",$dateUtcOrig);

	    // initialize realtimeArray
	    $realtimeArray = array();
	    for ($i=1; $i<59; $i++)
		    { $realtimeArray[$i]=0.0;}
	    $realtimeArray[1] = '-'; $realtimeArray[2] = '-'; $realtimeArray[12] = '-'; $realtimeArray[14] = '-'; $realtimeArray[15] = '-'; $realtimeArray[16] = '-'; $realtimeArray[17] = '-';
	    $realtimeArray[28] = '-'; $realtimeArray[30] = '-'; $realtimeArray[32] = '-'; $realtimeArray[34] = '-'; $realtimeArray[36] = '-'; $realtimeArray[38] = '-'; $realtimeArray[39] = '-';
	    $realtimeArray[52] = '-'; $realtimeArray[54] = '-';
	    $realtimeArray[4] = '0'; $realtimeArray[8] = '0'; $realtimeArray[13] = '0'; $realtimeArray[24] = '0'; $realtimeArray[40] = '0'; $realtimeArray[44] = '0'; $realtimeArray[46] = '0'; $realtimeArray[47] = '0'; $realtimeArray[49] = '0'; $realtimeArray[50] = '0'; $realtimeArray[51] = '0'; $realtimeArray[53] = '0'; $realtimeArray[58] = '0';
        if ($realtime_calculate_minmax)
            {
    		// only alldata sensors ( main )
    		$whereParam = "DATE(DateTime) = CURDATE()";
    	    $result = mysqli_query($con, "
    				SELECT  max(Tmax), min(Tmin), max(P), min(P), max(W), max(G)
    				FROM  alldata
    				WHERE ".$whereParam."
    				ORDER BY DateTime");
    		/*echo "	SELECT  max(Tmax), min(Tmin), max(P), min(P), max(W), max(G)
    				FROM  alldata
    				WHERE ".$whereParam."
    				ORDER BY alldata.DateTime<br>";*/
    		//echo "dberror=".mysqli_error($con)."<br>";
    		$i = 0;
    		while ($row = mysqli_fetch_array($result))
    		    {
    		    //$i++; echo $i;
    		    //print_r($row); echo "<br>";
    			if (!is_null($row['max(Tmax)'])) {$maxTc = convertor($row['max(Tmax)'],$dataTempUnits,"C"); $maxTf = convertor($row['max(Tmax)'],$dataTempUnits,"F");}
    			if (!is_null($row['min(Tmin)'])) {$minTc = convertor($row['min(Tmin)'],$dataTempUnits,"C"); $minTf = convertor($row['min(Tmin)'],$dataTempUnits,"F");}
    			if (!is_null($row['max(Tmax)'])) {$maxTorig = $row['max(Tmax)'];}
    			if (!is_null($row['min(Tmin)'])) {$minTorig = $row['min(Tmin)'];}
    
    			if (!is_null($row['max(P)']))    {$maxPhpa = convertor($row['max(P)'],$dataPressUnits,"hpa"); $maxPinhg = convertor($row['max(P)'],$dataPressUnits,"inhg");}
    			if (!is_null($row['min(P)']))    {$minPhpa = convertor($row['min(P)'],$dataPressUnits,"hpa"); $minPinhg = convertor($row['min(P)'],$dataPressUnits,"inhg");}
    			if (!is_null($row['max(P)']))    {$maxPorig = $row['max(P)'];      }
    			if (!is_null($row['min(P)']))    {$minPorig = $row['min(P)'];      }
    
    			if (!is_null($row['max(W)']))    {$maxWkmh = convertor($row['max(W)'],$dataWindUnits,"kmh"); $maxWmph = convertor($row['max(W)'],$dataWindUnits,"mph");}
    			if (!is_null($row['max(W)']))    {$maxWorig = $row['max(W)'];      }
    
    			if (!is_null($row['max(G)']))    {$maxGkmh = convertor($row['max(G)'],$dataWindUnits,"kmh"); $maxGmph = convertor($row['max(G)'],$dataWindUnits,"mph");}
    			if (!is_null($row['max(G)']))    {$maxGorig = $row['max(G)'];      }
    			//echo "maxPhpa=$maxPhpa #$dataPressUnits,hpa#<br>";
    			}	// while ($row = mysqli_fetch_array($result))    ( main )
    
    		// min/max time ( for all sensors )
    		$result = mysqli_query($con, "
    				SELECT  DateTime
    				FROM  alldata
    				WHERE Tmax = $maxTorig AND $whereParam
    				ORDER BY DateTime DESC
    				LIMIT 1
    				"
    		);
    		$maxTtime = "";
    		while ($row = mysqli_fetch_array($result)) {
    			$maxTtime = strtotime($row['DateTime']);
    			$maxTtime = date("H:i",$maxTtime);
    		}
    		$result = mysqli_query($con, "
    				SELECT  DateTime
    				FROM  alldata
    				WHERE Tmin = $minTorig AND $whereParam
    				ORDER BY DateTime DESC
    				LIMIT 1
    				"
    		);
    		$minTtime = "";
    		while ($row = mysqli_fetch_array($result)) {
    			$minTtime = strtotime($row['DateTime']);
    			$minTtime = date("H:i",$minTtime);
    		}
    		$result = mysqli_query($con, "
    				SELECT  DateTime
    				FROM  alldata
    				WHERE P = $maxPorig AND $whereParam
    				ORDER BY DateTime DESC
    				LIMIT 1
    				"
    		);
    		$maxPtime = "";
    		while ($row = mysqli_fetch_array($result)) {
    			$maxPtime = strtotime($row['DateTime']);
    			$maxPtime = date("H:i",$maxPtime);
    		}
    		$result = mysqli_query($con, "
    				SELECT  DateTime
    				FROM  alldata
    				WHERE P = $minPorig AND $whereParam
    				ORDER BY DateTime DESC
    				LIMIT 1
    				"
    		);
    		$minPtime = "";
    		while ($row = mysqli_fetch_array($result)) {
    			$minPtime = strtotime($row['DateTime']);
    			$minPtime = date("H:i",$minPtime);
    		}
    		$result = mysqli_query($con, "
    				SELECT  DateTime
    				FROM  alldata
    				WHERE W = $maxWorig AND $whereParam
    				ORDER BY DateTime DESC
    				LIMIT 1
    				"
    		);
    		$maxWtime = "";
    		while ($row = mysqli_fetch_array($result)) {
    			$maxWtime = strtotime($row['DateTime']);
    			$maxWtime = date("H:i",$maxWtime);
    		}
    		$result = mysqli_query($con, "
    				SELECT  DateTime
    				FROM  alldata
    				WHERE G = $maxGorig AND $whereParam
    				ORDER BY DateTime DESC
    				LIMIT 1
    				"
    		);
    		$maxGtime = "";
    		while ($row = mysqli_fetch_array($result)) 
    		    {
    			$maxGtime = strtotime($row['DateTime']);
    			$maxGtime = date("H:i",$maxGtime);
    		    }
            }   // if ($realtime_calculate_minmax)

	    // define fields
		if ($realtime_utc_time)
		    {
	    	$realtimeArray[1] = substr($dt[0],8,2).'/'.substr($dt[0],5,2).'/'.substr($dt[0],2,2);
	    	$realtimeArray[2] = $dt[1];
			}
		else
		    {
	    	$realtimeArray[1] = date("d/m/y",$utcTime+$offset);
	    	$realtimeArray[2] = date("H:i:s",$utcTime+$offset);
			}
		if ($realtime_metric_unit)
		    {
    	    if (isset($weather_data['tempc']))          $realtimeArray[3] = $weather_data['tempc']; $realtimeArray[15] = 'C';
    	    if (isset($weather_data['humidity']))       $realtimeArray[4] = $weather_data['humidity'];
    	    if (isset($weather_data['dewptc']))         $realtimeArray[5] = $weather_data['dewptc'];
    	    if (isset($weather_data['windspeedkmh']))   $realtimeArray[6] = $weather_data['windspeedkmh']; $realtimeArray[14] = 'km/h';
    	    if (isset($weather_data['windgustkmh']))    $realtimeArray[7] = $weather_data['windgustkmh'];
    	    if (isset($weather_data['maxdailygustkmh']))$realtimeArray[33] = $weather_data['maxdailygustkmh'];
    	    if (isset($weather_data['winddir']))        $realtimeArray[8] = $weather_data['winddir'];
    	    if (isset($weather_data['rainratemm']))     $realtimeArray[9] = $weather_data['rainratemm']; $realtimeArray[17] = 'mm';
    	    if (isset($weather_data['dailyrainmm']))    $realtimeArray[10] = $weather_data['dailyrainmm'];
    	    if (isset($weather_data['monthlyrainmm']))  $realtimeArray[20] = $weather_data['monthlyrainmm'];
    	    if (isset($weather_data['yearlyrainmm']))   $realtimeArray[21] = $weather_data['yearlyrainmm'];
    	    if (isset($weather_data['baromrelhpa']))    $realtimeArray[11] = $weather_data['baromrelhpa']; $realtimeArray[16] = 'hPa';
            if (isset($weather_data['tempinc']))        $realtimeArray[23] = $weather_data['tempinc'];
            if (isset($weather_data['humidityin']))     $realtimeArray[24] = $weather_data['humidityin'];
            if (isset($weather_data['windchillc']))     $realtimeArray[25] = $weather_data['windchillc'];
    	    if (isset($weather_data['uv']))             $realtimeArray[44] = $weather_data['uv'];
            if (isset($weather_data['solarradiation'])) $realtimeArray[46] = $weather_data['solarradiation'];
            if (isset($maxTc))                          $realtimeArray[27] = $maxTc;
            if (isset($maxTtime))                       $realtimeArray[28] = $maxTtime;
            if (isset($minTc))                          $realtimeArray[29] = $minTc;
            if (isset($minTtime))                       $realtimeArray[30] = $minTtime;
            if (isset($maxWkmh))                        $realtimeArray[31] = $maxWkmh;
            if (isset($maxWtime))                       $realtimeArray[32] = $maxWtime;
            if (isset($maxGkmh))                        $realtimeArray[33] = $maxGkmh;
            if (isset($maxGtime))                       $realtimeArray[34] = $maxGtime;
            if (isset($maxPhpa))                        $realtimeArray[35] = $maxPhpa;
            if (isset($maxPtime))                       $realtimeArray[36] = $maxPtime;
            if (isset($minPhpa))                        $realtimeArray[37] = $minPhpa;
            if (isset($minPtime))                       $realtimeArray[38] = $minPtime;
		    }
		else
		    {
    	    if (isset($weather_data['tempf']))          $realtimeArray[3] = $weather_data['tempf']; $realtimeArray[15] = 'F';
    	    if (isset($weather_data['humidity']))       $realtimeArray[4] = $weather_data['humidity'];
    	    if (isset($weather_data['dewptf']))         $realtimeArray[5] = $weather_data['dewptf'];
    	    if (isset($weather_data['windspeedmph']))   $realtimeArray[6] = $weather_data['windspeedmph']; $realtimeArray[14] = 'mph';
    	    if (isset($weather_data['windgustmph']))    $realtimeArray[7] = $weather_data['windgustmph'];
    	    if (isset($weather_data['maxdailygust']))   $realtimeArray[33] = $weather_data['maxdailygust'];
    	    if (isset($weather_data['winddir']))        $realtimeArray[8] = $weather_data['winddir'];
    	    if (isset($weather_data['rainratein']))     $realtimeArray[9] = $weather_data['rainratein']; $realtimeArray[17] = 'in';
    	    if (isset($weather_data['dailyrainin']))    $realtimeArray[10] = $weather_data['dailyrainin'];
    	    if (isset($weather_data['monthlyrainin']))  $realtimeArray[20] = $weather_data['monthlyrainin'];
    	    if (isset($weather_data['yearlyrainin']))   $realtimeArray[21] = $weather_data['yearlyrainin'];
    	    if (isset($weather_data['baromrelin']))     $realtimeArray[11] = $weather_data['baromrelin']; $realtimeArray[16] = 'in';
            if (isset($weather_data['tempinf']))        $realtimeArray[23] = $weather_data['tempinf'];
            if (isset($weather_data['humidityin']))     $realtimeArray[24] = $weather_data['humidityin'];
            if (isset($weather_data['windchillf']))     $realtimeArray[25] = $weather_data['windchillf'];
    	    if (isset($weather_data['uv']))             $realtimeArray[44] = $weather_data['uv'];
            if (isset($weather_data['solarradiation'])) $realtimeArray[46] = $weather_data['solarradiation'];
            if (isset($maxTf))                          $realtimeArray[27] = $maxTf;
            if (isset($maxTtime))                       $realtimeArray[28] = $maxTtime;
            if (isset($minTf))                          $realtimeArray[29] = $minTf;
            if (isset($minTtime))                       $realtimeArray[30] = $minTtime;
            if (isset($maxWmph))                        $realtimeArray[31] = $maxWmph;
            if (isset($maxWtime))                       $realtimeArray[32] = $maxWtime;
            if (isset($maxGmph))                        $realtimeArray[33] = $maxGmph;
            if (isset($maxGtime))                       $realtimeArray[34] = $maxGtime;
            if (isset($maxPinhg))                       $realtimeArray[35] = $maxPinhg;
            if (isset($maxPtime))                       $realtimeArray[36] = $maxPtime;
            if (isset($minPinhg))                       $realtimeArray[37] = $minPinhg;
            if (isset($minPtime))                       $realtimeArray[38] = $minPtime;
		    }

	    $realtimeString = implode(' ',$realtimeArray);
	    file_put_contents($json_data_logdir . "/" . "realtime.txt",$realtimeString);
	    $LogEcowitt['info'][] = "realtime.txt = ".$realtimeString;
        }
	
	# Pack data into json format
	$weather_data_json = json_encode($weather_data);
	$weather_data_orig_json = json_encode($weather_data_orig);

	# Write json stream to logfile
	if ( $json_data_log ) 
    	{
	    $json_data_logfile = $json_data_logdir . "/" . $device . ".json";
		$file = fopen($json_data_logfile, 'w');
		//fwrite($file, $weather_data_orig_json);
		fwrite($file, $weather_data_json);
		fclose($file);
    	}

	# Write stream to csvfile
	if ($txt_data_log)
	    {
	    $csv_data_logfile = $csv_data_logdir . "/" . $device . "_" . $date_txt . ".csv";
		if (!file_exists($csv_data_logfile)) 
		    {
			$data = json_decode($weather_data_json);
			foreach($data as $key => $value) 
			    {$string .= $key . ',';}
			$string .= "\n";
			file_put_contents($csv_data_logfile, $string, FILE_APPEND);
		    }

		$file = fopen($csv_data_logfile, 'a');
		fputcsv($file, $weather_data);
		fclose($file);
	    }

	for ($i=1; $i <= $nForwarders; $i++)
		{
		if ($forwardData[$i] == "ecowitt")
	       {
           $url = $forwardServer[$i];
           $fields = $weather_data_orig;           // post fields
           $postvars = http_build_query($fields);  // build the urlencoded data
           $postvars=str_replace("%3A",":",$postvars);
           $ch = curl_init();

           // set the url, number of POST vars, POST data, port
           curl_setopt($ch, CURLOPT_URL, $url);
           curl_setopt($ch, CURLOPT_POST, count($fields));
           curl_setopt($ch, CURLOPT_POSTFIELDS, $postvars);
           if (!empty($forwardPort[$i])) {curl_setopt($ch, CURLOPT_PORT, $forwardPort[$i]);}
           $result = curl_exec($ch);   // execute post
           $LogEcowitt['info'][] = "Forward".$i." ecowitt format";
           $LogEcowitt['info'][] = "Forward".$i." url -> ".$url;
           if (!empty($forwardPort[$i])) {$LogEcowitt['info'][] = "Forward1 port -> ".$forwardPort[$i];}
           $LogEcowitt['info'][] = "Forward".$i." nfields -> ".count($fields);
           $LogEcowitt['info'][] = "Forward".$i." result -> ".$result;
           if (curl_error($ch)) {$LogEcowitt['info'][] = "Forward".$i." error -> ".curl_error($ch);}
           curl_close($ch);
	       }

		if ( ($forwardData[$i] == "wu") || ($forwardData[$i] == "wuext") )
	       {
           $url = $forwardServer[$i];
		   $wdf = "";
		   foreach($weather_data_wu_forward as $key => $value)
		      {$wdf .= '&' . $key . '=' . $value;}
		   if ($forwardData[$i] == "wuext")
		   	  {
		   	  foreach($weather_data_wuext_forward as $key => $value)
			     {$wdf .= '&' . $key . '=' . $value;}
		      }
		   @$result = file_get_contents($url  . "?ID=" . $forwardUsername[$i] . "&PASSWORD=" . $forwardPassword[$i] . "&action=updateraw" . $wdf);

           $a = strpos($url,"?");
           if ($a !== false) {$url = substr($url,0,$a); }
           $LogEcowitt['info'][] = "Forward".$i." wu format";
           $LogEcowitt['info'][] = "Forward".$i." url -> ".$url . "&action=updateraw" . $wdf;
           //$LogEcowitt['info'][] = "Forward".$i." url -> ".$url . "?ID=" . $forwardUsername[$i] . "&PASSWORD=" . $forwardPassword[$i] . "&action=updateraw" . $wdf;
           $LogEcowitt['info'][] = "Forward".$i." result -> ".$result;
	       }

		if ($forwardData[$i] == "awekas")
	       {
           $url = $forwardServer[$i];
		   $wdf = "";
		   foreach($weather_data_awekas_forward as $key => $value)
			  {$wdf .= '&' . $key . '=' . $value;}
		   $wdf = substr($wdf,1);
		   $awekas_date = date("d.m.Y",strtotime($weather_data['dateutc']));
		   $awekas_time = date("H:i",strtotime($weather_data['dateutc']));
		   @$awekas_var0 = $forwardUsername[$i].";".md5($forwardPassword[$i]).";";   // 1-2
		   @$awekas_var  = $awekas_date.";".$awekas_time.";".$weather_data['tempc'].";";  // 3-5
		   @$awekas_var .= $weather_data['humidity'].";".$weather_data['baromrelhpa'].";".$weather_data['dailyrainmm'].";".$weather_data['windspeedkmh'].";".$weather_data['winddir'].";";   // 6-10
		   @$awekas_var .= ";;;en;;";    // 11-15
		   @$awekas_var .= $weather_data['windgustkmh'].";".$weather_data['solarradiation'].";".$weather_data['uv'].";;;";    // 16-20
		   @$awekas_var .= $weather_data['tf_ch1'].";".$weather_data['rainratemm'].";meteot_2.15;;";    // 21-25
		   @$result = file_get_contents($url  . "?" . $wdf . "&val=" . $awekas_var0 . $awekas_var);

           $LogEcowitt['info'][] = "Forward".$i." awekas format";
           $LogEcowitt['info'][] = "Forward".$i." url -> ".$url . "?" . $wdf . "&val=" . $awekas_var;
           $LogEcowitt['info'][] = "Forward".$i." result -> ".$result;
	       }

		if ($forwardData[$i] == "weatherwizz")
	       {
           $url = $forwardServer[$i];
		   $wdf = "";
		   foreach($weather_data_weatherwizz_forward as $key => $value)
		      {$wdf .= '&' . $key . '=' . $value;}
		   @$result = file_get_contents($url  . "?stationid=" . $forwardUsername[$i] . "&auth=" . $forwardPassword[$i] . $wdf);

           $a = strpos($url,"?");
           if ($a !== false) {$url = substr($url,0,$a); }
           $LogEcowitt['info'][] = "Forward".$i." weatherwizz format";
           $LogEcowitt['info'][] = "Forward".$i." url -> ".$url . $wdf;
           //$LogEcowitt['info'][] = "Forward".$i." url -> ".$url . "?ID=" . $forwardUsername[$i] . "&PASSWORD=" . $forwardPassword[$i] . "&action=updateraw" . $wdf;
           $LogEcowitt['info'][] = "Forward".$i." result -> ".$result;
	       }
	    }

	generateLogEcowitt($base, $LogEcowitt, "ecowittLog.txt");
	echo "Success. Update done\n";
	
    ############################################################################
    // Functions
    ############################################################################

    function generateLogEcowitt($base, $apiLog, $fileName)
        {
        //global $apiLog;
        //global $base;
        // create log and exit update script 
        $apiLog['info'][] = "Generating log file cache/".$fileName;
        $updateLog = "";
        foreach($apiLog['info'] as $info)
            {$updateLog .= $info."\n";}
        $updateLog .= "\nERRORS:\n\r";
        if(isset($apiLog['error']))
            {
            foreach($apiLog['error'] as $error)
                {$updateLog .= $error."\n";}
            }
        file_put_contents($base."cache/".$fileName,$updateLog);
        }
        
    function subst_eval($se)
        {
        global $weather_data_forward;
        global $Log;
        
        $se1 = explode("#", $se);
        $i = 1;
        while ($i < count($se1)):
            $a = $se1[$i];
            $se1[$i] = $weather_data_forward[$se1[$i]];
            //$LogEcowitt['info'][] = "subst_eval -> ".$i." ; ".$a." ; ".$se1[$i];
            $i += 2;
            endwhile;
        $result = "";
        for ($i=0; $i<count($se1); $i++)
            {
            $result .= $se1[$i];
            }
        //$LogEcowitt['info'][] = "subst_eval --> ".$se."=".$result;
        @eval("\$result1 = $result;");
        //$LogEcowitt['info'][] = "subst_eval --> ".$se."=".$result1;
        return $result1;
        }

    // Dew point calculation
    // accepts temperature [C] and humidity [%]
    // returns dew point [C]
    function dewpoint1($dewT,$dewH){
        $calcD = round(((pow(($dewH/100), 0.125))*(112+0.9*$dewT)+(0.1*$dewT)-112),1);
        return $calcD;
    }

?>

